#include <PalmOS.h>
#include <PalmCompatibility.h>
#include <SonyCLIE.h>
#include <TrgChars.h>
#include <vga.h>
#include <SonySilkLib.h>
#include <BmpGlue.h> // for BmpGlueGetDimension
// #include <Window.h>
#include "jdreader_res.h"

#include "MyUtility.h"

#ifndef nil
#define nil 0
#endif

// Change here for HandEra SDK update
#define TRGJogUpChar vchrPrevField
#define TRGJogDownChar vchrNextField
#define TRGJogPressChar chrCarriageReturn

#define jdocScreenExtend 1
#define jdocScreenRotate 2

typedef enum {
	STORAGE_INTERNAL =0,
	STORAGE_CARD
} StorageEnum;

typedef enum {
	ACTION_NONE=0,
	ACTION_PREVIOUS_LINE,
	ACTION_NEXT_LINE,
	ACTION_PREVIOUS_SCREEN,
	ACTION_NEXT_SCREEN,
	ACTION_PREVIOUS_TOPIC,
	ACTION_NEXT_TOPIC,
	ACTION_PREVIOUS_DOCUMENT,
	ACTION_NEXT_DOCUMENT,
	ACTION_DELETE_WITHOUT_CONFIRM,
	ACTION_DELETE_WITH_CONFIRM,
	ACTION_CLOSE_DOCUMENT
} actionEnum;

typedef enum {
	BUTTON_TAP_UPPER=0,
	BUTTON_TAP_LOWER,
	BUTTON_UP_BUTTON,
	BUTTON_DOWN_BUTTON,
	BUTTON_DATE_BUTTON,
	BUTTON_ADDRESS_BUTTON,
	BUTTON_TODO_BUTTON,
	BUTTON_MEMO_BUTTON,
	BUTTON_JOG_UP,
	BUTTON_JOG_DOWN,
	BUTTON_JOG_RELEASE,
	BUTTON_JOG_PAGE_UP,
	BUTTON_JOG_PAGE_DOWN,
	BUTTON_NAVIGATOR_RIGHT,
	BUTTON_NAVIGATOR_LEFT,
	BUTTON_NAVIGATOR_SELECT
} buttonEnum;

#define NUM_ACTION 12  // 2 for screen tap, 6 for buttons 5 for Jogs
#define NUM_BUTTON 13 

#define MS_TEXT_PATH "/palm/programs/text/"
#define CARD_TEXT_PATH "/texts/"

#define MAX_DOCUMENTS 512
#define noSelected -1


typedef	struct trecord0 {
		Word	compression_type;
		Word	spare;
		DWord	story_length;
		Word	num_records;
		Word	reclen;		// normally 0x1000;
		UInt32	dummy;
} Record0Type;

typedef struct {
	UInt				currentView;
	LocalID				currentDBlocalId;
	UInt				currentDBcard;
	ULong				currentCreateDate;
	UInt				currentRecord;
	UInt				topVisibleLine;	// View form
	int				orientation; // Screen orientation
	int 				button_action[NUM_BUTTON ]; // 0..up 1..down 2..upscreen 3..downscreen 4..7  Hard Button 8..12 JOG
	FontID			font;
	StorageEnum				storage_mode; 
	char file_name[256];
} JDocPreferenceType;

// Application database management defines.
#define JDocAppType		'JDoc'			// type for application. J-DOC
#define DocDBCreator	'REAd'			// name for application 'DOC'
#define DocDBType		'TEXt'			// type for application 'DOC'
#define JDocVersionNum					4

/***********************************************************************
 * Global variables for this module
 **********************************************************************/
// static int				Orientation;		// 0 for Normal 1,2,3 for Rotated * 90
static LocalID				CurrentDBlocalId;			
static UInt				CurrentDBcard;
static ULong			CurrentRecord;
static UInt				TopVisibleLine = 0;
static FontID		DisplayFont;
static int LineSpace;
static int ScrollLines;
static int PageLine;
static DmOpenRef		CurrentDB;				// P5. handle for database
static int				CurrentDBNumber;	// sequential number startingfrom 0
static ULong			CurrentCreateDate;
static ULong			TotalBytes;
static ULong			TotalRecords;
static VoidHand			dataHandle;
static int				CompressionMode;	// 0..no compression 1.. Pat's compression
static Boolean		SkipJogRelease=false; // set to true when JogPageUp or JogPageDown received
// flag for MainView to do something
static int	NextOpenDocument;
static int	GotoBottomFlag;
static int	DeleteDBNumber;

Boolean NeedResearchDocumentList = false;

static UInt				TopVisibleRecord =0;
// static WinHandle	dispWin;
static int TailBufferRecordNum;
static CharPtr TailBuffer;
static int 	button_action[NUM_BUTTON ];
BitmapType *BackBitmap, *WorkBitmap;
WindowType *BackWindow, *WorkWindow;
int PageLines,topOfPage;
static	int 		TotalDocuments;
MemHandle lineInfoHandle;
MemHandle operation;
Boolean  IgnoreJogRelease;
Boolean	 CursorDisplayed;

Boolean MediaExists; // MS(Volume Type)  is in?
Boolean MediaWritable; // Is it writable and not write-protected?
UInt16 volumeRefNum; // Volume reference number for MS or SD card

JDocPreferenceType prefs;	

int numMSFiles;
MemHandle MSFileListH;
FontID MainFormFont;
int TableRows; // Used in MainForm; Change dynamically

// Following if Record and TopOfPage on the screen.
Int16	displayedTopOfPage;
Int16	displayedRecord;


char **parray;
Handle parrayH;


struct lineinfo {
	int start;
	short	chars;
	Boolean top_of_line;
} *lineInfoPtr;

typedef struct ducument_info {
		UInt16 card_no;
		LocalID	local_id;
		char name[33];
		ULong size;
} document_infoType, *document_infoP;

typedef struct{
	char file_name[256];
} MSFileType;

char card_path[64]; // Default Directory for text file
MSFileType *MSFileListP=0;
FileRef CurrentTextFile; // FileRef of current opened text

static document_infoP doc_infos[MAX_DOCUMENTS];
char DocumentName[128];

static Boolean CardStateChanged; // To show MS is removed or insterted
Boolean DisplayStateChanged; // To show Display Extent is changed.

MachineFunctionType MachineFunctionality; 
UInt16 hrRefNum;
Int16 screenWidth, screenHeight; // Pixels of Display
Int16 campusWidth, campusHeight; // Pixels of virtual screen( When rotated, this differs from screenWidth,screenHeight)
Int16 gScale; // Screen density  1 for LoRes , 2 for HiRes
MemPtr saveJogAsisstMaskPtr;
UInt16 jogAsisstMask[6] = { 0x0001, 2,  1000 , 0xbf, 1200, 0xbf};
Int16 new_orientation; // Variable passed to EditView from ScreenOrientationView
UInt16 SLKref; // Library reference for Sony Silk Library

typedef struct {
	ColorTableType clut;
	RGBColorType rgb[2];
} MonochromeCLUTType;

MonochromeCLUTType mono_clut;

#define msRemovedEvent sysEventFirstUserEvent

/***********************************************************************
 * Prototypes for internal functions
 **********************************************************************/
Boolean StartApplication(void);
void StopApplication(void);								// P5. clean up before app exit
VoidPtr GetObjectPtr(Int objectID);					// P10. get ptr to object in active form
void EditViewDoMenuCommand(Int command);					// P4. perform commands for edit form menu
Boolean EditViewRetrieveData();					// P5. get data from application database
Boolean EditViewHandleEvent(EventPtr event);
void MainViewInit(void);									// P7. initialize data, objects for the main form
Boolean MainViewHandleEvent(EventPtr event);
Boolean ApplicationHandleEvent(EventPtr event);	// P3. handle form load events
void EventLoop(void);
void TotalDraw();
void MainViewDrawRecordInBounds(CharPtr, RectanglePtr);
void ReadOneRecord();
void CloseDocument(void);
void OptionFormInit(void);
void TransferWindow(int);
void MainViewUpdateScrollers(void);
void MainViewDoMenuCommand(int);
void GotoNextTopic(void);
void GotoPreviousTopic(void);
Int CompFunc( void *p1,void *p2,long other);
void NextDocument(void);
void PreviousDocument(void);
Boolean HardKeyReserved(EventType *evt);
void InitialUse(void);
void MainViewSetDocumentChoice(int choice );
Boolean CheckForStorage();
Boolean DummyHandleEvent(EventPtr event);
Boolean IsHardKeyReserved(EventType *evt);
